@@ -1,10 +1,10 @@
 ##----------------------------------------------------------------------------
-## Module Generic - ~/lib/Module/Generic/Number.pm
-## Version v1.0.1
+## Module Generic - ~/lib/Generic/Number.pm
+## Version v1.0.2
 ## Copyright(c) 2021 DEGUEST Pte. Ltd.
 ## Author: Jacques Deguest <jack@deguest.jp>
 ## Created 2021/03/20
-## Modified 2021/04/21
+## Modified 2021/12/06
 ## All rights reserved
 ## 
 ## This program is free software; you can redistribute  it  and/or  modify  it
@@ -23,9 +23,10 @@ BEGIN
     use Module::Generic::Scalar;
     use Number::Format;
     use Nice::Try;
-    use Regexp::Common qw( number );
     use POSIX ();
-    our( $VERSION ) = 'v1.0.1';
+    use Regexp::Common qw( number );
+    use Scalar::Util ();
+    our( $VERSION ) = 'v1.0.2';
 };
 
 use overload (
@@ -480,10 +481,11 @@ sub init
     $self->{default} = $DEFAULT;
     $self->{_init_strict_use_sub} = 1;
     $self->SUPER::init( @_ );
-    my $default = $self->default;
+    $self->{_original} = $num;
+     my $default = $self->default;
     # $self->message( 3, "Getting current locale" );
     my $curr_locale = POSIX::setlocale( &POSIX::LC_ALL );
-    ## $self->message( 3, "Current locale is '$curr_locale'" );
+    # $self->message( 3, "Current locale is '$curr_locale'" );
     if( $self->{lang} )
     {
         # $self->message( 3, "Language requested '$self->{lang}'." );
@@ -597,18 +599,18 @@ sub init
     foreach my $prop ( keys( %$map ) )
     {
         my $ref = $map->{ $prop };
-        ## Already set by user
+        # Already set by user
         next if( CORE::length( $self->{ $prop } ) );
         foreach my $lconv_prop ( @$ref )
         {
             if( CORE::defined( $default->{ $lconv_prop } ) )
             {
-                ## Number::Format bug RT #71044 when running on Windows
-                ## https://rt.cpan.org/Ticket/Display.html?id=71044
-                ## This is a workaround when values are lower than 0 (i.e. -1)
+                # Number::Format bug RT #71044 when running on Windows
+                # https://rt.cpan.org/Ticket/Display.html?id=71044
+                # This is a workaround when values are lower than 0 (i.e. -1)
                 if( CORE::exists( $numerics->{ $lconv_prop } ) && 
                     CORE::length( $default->{ $lconv_prop } ) && 
-                    ## It may be a non-numeric value which would wreak the following condition
+                    # It may be a non-numeric value which would wreak the following condition
                     $default->{ $lconv_prop } =~ /\d+/ &&
                     $default->{ $lconv_prop } < 0 )
                 {
@@ -624,63 +626,12 @@ sub init
         }
     }
     
-    # $Number::Format::DEFAULT_LOCALE->{int_curr_symbol} = 'EUR';
-    try
-    {
-        ## Those are unsupported by Number::Format
-        my $skip =
-        {
-        int_n_cs_precedes => 1,
-        int_p_cs_precedes => 1,
-        int_n_sep_by_space => 1,
-        int_p_sep_by_space => 1,
-        int_n_sign_posn => 1,
-        int_p_sign_posn => 1,
-        };
-        my $opts = {};
-        foreach my $prop ( CORE::keys( %$map ) )
-        {
-            ## $self->message( 3, "Checking property \"$prop\" value \"", overload::StrVal( $self->{ $prop } ), "\" (", $self->$prop->defined ? 'defined' : 'undefined', ")." );
-            my $prop_val;
-            if( $self->$prop->defined )
-            {
-                $prop_val = $self->$prop;
-            }
-            ## To prevent Number::Format from defaulting to property values not in sync with ours
-            ## Because it seems the POSIX::setlocale only affect one module
-            else
-            {
-                $prop_val = '';
-            }
-            ## $self->message( 3, "Using property \"$prop\" value \"$prop_val\" (", CORE::defined( $prop_val ) ? 'defined' : 'undefined', ") [ref=", ref( $prop_val ), "]." );
-            ## Need to set all the localeconv properties for Number::Format, because it uses mon_thousand_sep intsead of just thousand_sep
-            foreach my $lconv_prop ( @{$map->{ $prop }} )
-            {
-                CORE::next if( CORE::exists( $skip->{ $lconv_prop } ) );
-                ## Cannot be undefined, but can be empty string
-                $opts->{ $lconv_prop } = "$prop_val";
-                if( !CORE::length( $opts->{ $lconv_prop } ) && CORE::exists( $numerics->{ $lconv_prop } ) )
-                {
-                    $opts->{ $lconv_prop } = $numerics->{ $lconv_prop };
-                }
-            }
-        }
-        ## $self->message( 3, "Using following options for Number::Format: ", sub{ $self->dumper( $opts ) } );
-        no warnings qw( uninitialized );
-        $self->{_fmt} = Number::Format->new( %$opts );
-        use warnings;
-    }
-    catch( $e )
-    {
-        ## $self->message( 3, "Error trapped in creating a Number::Format object: '$e'" );
-        return( $self->error( "Unable to create a Number::Format object: $e" ) );
-    }
-    $self->{_original} = $num;
     try
     {
         if( $num !~ /^$RE{num}{real}$/ )
         {
-            $self->{_number} = $self->{_fmt}->unformat_number( $num );
+            my $fmt = $self->_get_formator;
+            $self->{_number} = $fmt->unformat_number( $num );
         }
         else
         {
@@ -749,7 +700,7 @@ sub compute
         return( Module::Generic::Scalar->new( $res ) ) if( $opts->{type} eq 'scalar' );
         return( Module::Generic::Infinity->new( $res ) ) if( POSIX::isinf( $res ) );
         return( Module::Generic::Nan->new( $res ) ) if( POSIX::isnan( $res ) );
-        ## undef may be returned for example on platform supporting NaN when using <=>
+        # undef may be returned for example on platform supporting NaN when using <=>
         return( $self->clone( $res ) ) if( defined( $res ) );
         return;
     }
@@ -788,7 +739,7 @@ sub format
     my $num  = $self->{_number};
     ## If value provided was undefined, we leave it undefined, otherwise we would be at risk of returning 0, and 0 is very different from undefined
     return( $num ) if( !defined( $num ) );
-    my $fmt = $self->{_fmt};
+    my $fmt = $self->_get_formator;
     try
     {
         ## Amazingly enough, when a precision > 0 is provided, format_number will discard it if the number, before formatting, did not have decimals... Then, what is the point of formatting a number then?
@@ -813,7 +764,7 @@ sub format_bytes
     my $num  = $self->{_number};
     ## See comment in format() method
     return( $num ) if( !defined( $num ) );
-    my $fmt = $self->{_fmt};
+    my $fmt = $self->_get_formator;
     try
     {
         ## return( $fmt->format_bytes( $num, @_ ) );
@@ -838,7 +789,7 @@ sub format_money
     my $num  = $self->{_number};
     ## See comment in format() method
     return( $num ) if( !defined( $num ) );
-    my $fmt = $self->{_fmt};
+    my $fmt = $self->_get_formator;
     try
     {
         ## Even though the Number::Format instantiated is set with a currency symbol, 
@@ -863,7 +814,7 @@ sub format_negative
     my $num  = $self->{_number};
     ## See comment in format() method
     return( $num ) if( !defined( $num ) );
-    my $fmt = $self->{_fmt};
+    my $fmt = $self->_get_formator;
     try
     {
         my $new = $self->format;
@@ -887,7 +838,7 @@ sub format_picture
     my $num  = $self->{_number};
     ## See comment in format() method
     return( $num ) if( !defined( $num ) );
-    my $fmt = $self->{_fmt};
+    my $fmt = $self->_get_formator;
     try
     {
         ## return( $fmt->format_picture( $num, @_ ) );
@@ -901,7 +852,7 @@ sub format_picture
     }
 }
 
-sub formatter { return( shift->_set_get_object( 'formatter', 'Number::Format', @_ ) ); }
+sub formatter { return( shift->_set_get_object( '_fmt', 'Number::Format', @_ ) ); }
 
 ## https://stackoverflow.com/a/483708/4814971
 sub from_binary
@@ -1061,7 +1012,7 @@ sub round2
     my $num  = $self->{_number};
     ## See comment in format() method
     return( $num ) if( !defined( $num ) );
-    my $fmt = $self->{_fmt};
+    my $fmt = $self->_get_formator;
     try
     {
         ## return( $fmt->round( $num, @_ ) );
@@ -1108,7 +1059,7 @@ sub unformat
     return if( !defined( $num ) );
     try
     {
-        my $num2 = $self->{_fmt}->unformat_number( $num );
+        my $num2 = $self->_get_formator->unformat_number( $num );
         my $clone = $self->clone;
         $clone->{_original} = $num;
         $clone->{_number} = $num2;
@@ -1141,6 +1092,64 @@ sub _func
     return( $self->clone( $res ) );
 }
 
+sub _get_formator
+{
+    my $self = shift( @_ );
+    return( $self->{_fmt} ) if( $self->{_fmt} );
+    # $Number::Format::DEFAULT_LOCALE->{int_curr_symbol} = 'EUR';
+    try
+    {
+        ## Those are unsupported by Number::Format
+        my $skip =
+        {
+        int_n_cs_precedes => 1,
+        int_p_cs_precedes => 1,
+        int_n_sep_by_space => 1,
+        int_p_sep_by_space => 1,
+        int_n_sign_posn => 1,
+        int_p_sign_posn => 1,
+        };
+        my $opts = {};
+        foreach my $prop ( CORE::keys( %$map ) )
+        {
+            ## $self->message( 3, "Checking property \"$prop\" value \"", overload::StrVal( $self->{ $prop } ), "\" (", $self->$prop->defined ? 'defined' : 'undefined', ")." );
+            my $prop_val;
+            if( $self->$prop->defined )
+            {
+                $prop_val = $self->$prop;
+            }
+            ## To prevent Number::Format from defaulting to property values not in sync with ours
+            ## Because it seems the POSIX::setlocale only affect one module
+            else
+            {
+                $prop_val = '';
+            }
+            ## $self->message( 3, "Using property \"$prop\" value \"$prop_val\" (", CORE::defined( $prop_val ) ? 'defined' : 'undefined', ") [ref=", ref( $prop_val ), "]." );
+            ## Need to set all the localeconv properties for Number::Format, because it uses mon_thousand_sep intsead of just thousand_sep
+            foreach my $lconv_prop ( @{$map->{ $prop }} )
+            {
+                CORE::next if( CORE::exists( $skip->{ $lconv_prop } ) );
+                ## Cannot be undefined, but can be empty string
+                $opts->{ $lconv_prop } = "$prop_val";
+                if( !CORE::length( $opts->{ $lconv_prop } ) && CORE::exists( $numerics->{ $lconv_prop } ) )
+                {
+                    $opts->{ $lconv_prop } = $numerics->{ $lconv_prop };
+                }
+            }
+        }
+        ## $self->message( 3, "Using following options for Number::Format: ", sub{ $self->dumper( $opts ) } );
+        no warnings qw( uninitialized );
+        $self->{_fmt} = Number::Format->new( %$opts );
+        use warnings;
+        return( $self->{_fmt} );
+    }
+    catch( $e )
+    {
+        ## $self->message( 3, "Error trapped in creating a Number::Format object: '$e'" );
+        return( $self->error( "Unable to create a Number::Format object: $e" ) );
+    }
+}
+
 sub _set_get_prop
 {
     my $self = shift( @_ );
@@ -1148,14 +1157,15 @@ sub _set_get_prop
     if( @_ )
     {
         my $val = shift( @_ );
-        $val = $val->scalar if( $self->_is_object( $val ) && $val->isa( 'Module::Generic::Scalar' ) );
+        # $val = $val->scalar if( $self->_is_object( $val ) && $val->isa( 'Module::Generic::Scalar' ) );
+        $val = "$val" if( CORE::defined( $val ) );
         ## $self->message( 3, "Setting value \"$val\" (", defined( $val ) ? 'defined' : 'undefined', ") for property \"$prop\"." );
         ## I do not want to set a default value of '' to $self->{ $prop } because if its value is undef, it should remain so
         no warnings 'uninitialized';
         if( !CORE::defined( $val ) || ( CORE::defined( $val ) && $val ne $self->{ $prop } ) )
         {
             $self->_set_get_scalar_as_object( $prop, $val );
-            ## If an error was set, we return nothing
+            # If an error was set, we return nothing
             $self->formatter( $self->new_formatter ) || return;
         }
     }
@@ -1166,7 +1176,7 @@ AUTOLOAD
 {
     my( $method ) = our $AUTOLOAD =~ /([^:]+)$/;
     my $self = shift( @_ ) || return;
-    my $fmt_obj = $self->{_fmt} || return;
+    my $fmt_obj = $self->_get_formator || return;
     my $code = $fmt_obj->can( $method );
     if( $code )
     {
